from Products.Archetypes.Field import *
from Products.Archetypes.Widget import *
from Products.Archetypes.public import DisplayList
from Widgets import *
try:
    from validation import validation
except ImportError:
    from Products.validation import validation

class USAddressField(TextField):
    """A field that stores strings"""
    _properties = TextField._properties.copy()
    _properties.update({ 'widget':AddressWidget
                        , 'validators':('isUSAddress',)
                        })

class LinkField(StringField):
    """A field that stores strings"""
    _properties = StringField._properties.copy()
    _properties.update({ 'widget':LinkWidget
                        ,'validators':('isURL',)
                        })

class USPhoneField(StringField):
    """A field that stores strings"""
    _properties = StringField._properties.copy()
    _properties.update({ 'widget':USPhoneWidget,
        'validators':('isFormattedUSPhone',),
        })

class EmailField(StringField):
    """A field that stores strings"""
    _properties = StringField._properties.copy()
    _properties.update({ 'widget':EmailWidget
                        , 'validators':('isEmail',)
                        })

class InstructionField(ObjectField):
    """Just help"""
    _properties = ObjectField._properties.copy()
    _properties.update({ 'widget':InstructionWidget })


class SimpleDataGridField(LinesField):
    """A lines field with embedded vertical bars for fields.
    
    Fields are just strings, with no internal validation, etc.
    """
    _properties = LinesField._properties.copy()
    _properties.update({ 'widget':SimpleDataGridWidget ,
                         'delimiter':'|' })


    def get(self, instance, **kwargs):
        data = LinesField.get(self, instance, **kwargs)
        return data


    def getAsDisplayList(self, instance, **kwargs):
        data = self.get(instance, **kwargs)
        data = ([ d.split(self.delimiter) for d in data ])
        return DisplayList(data)
        
#-------- dyndoc helper classes

from AccessControl import getSecurityManager
from TAL.TALDefs import TALESError
from Products.PageTemplates import PageTemplate
from Products.PageTemplates.ZopePageTemplate import SecureModuleImporter
from Acquisition import Implicit, aq_parent, aq_inner
from Persistence import Persistent
from ExtensionClass import Base

class ddPageTemplate(PageTemplate.PageTemplate, Implicit, Persistent):
    def pt_getContext(self):
        root = self.getPhysicalRoot()
        template = aq_parent(self)
        here = aq_parent(template)
        c = {'template': template,
             'options': {},
             'nothing': None,
             'request': None,
             'root': root,
             'request': getattr(root, 'REQUEST', None),
             'modules': SecureModuleImporter,
             'here': here,
             'context': here,
             'container': aq_inner(here),
             }
        return c

    def __setstate__(self, state):
        ddPageTemplate.inheritedAttribute('__setstate__')(self, state)
        self._cook()

#class Macros(Base):
#
# taken from CMFDynamicDocument
#
# should be easy to allow dyndocs to be used as a macro (uncomment
# this and stuff below), but not tested/not interested in this.

#    """
#    Evaluate to the macros collection of the contained page template.
#    """
#    def __of__( self, doc):
#        return doc.get_pt().macros

class DynamicField(TextField):
    """Just help"""
    _properties = ObjectField._properties.copy()
    _properties.update({ 'widget':TextAreaWidget, 'default_output_type':'text/html' })

    #macros = Macros()
    #security.declareProtected(CMFCorePermissions.View, 'macros')

    def get_unwrapped(self, instance):
        """
        return unwrapped PageTemplate object
        """

        #import pdb
        #pdb.set_trace()
        #raise "foo",dir(fldname)
        try:
            return getattr(instance, self.getName()).__of__(instance)
        except:
            setattr(instance, self.getName(), ddPageTemplate())
            return getattr(instance, self.getName()).__of__(instance)
    
    def getRaw(self, instance, **kwargs):
        """Get raw"""

        # not sure if there's a better way to do this:
        # if this is our first access (ie, we haven't used property
        # before, then init to page template)
        # probably better handled in object's __init__ or addXXXX function,
        # both not sure how to handle __init__ properly, and
        # addXXXX function would make user of this fieldtype have to
        # create one for every class that used this field type.
        
        if not hasattr(instance, self.getName()):
            setattr(instance, self.getName(), ddPageTemplate())
        else:   
            return self.get_unwrapped(instance).read()
        
    def set(self, instance, value, **kwargs):
        """
          Edit the document
        """
        pt = self.get_unwrapped(instance)
        pt.write(value)

    def get(self, instance, mimetype=None, raw=0, **kwargs):
        """Return"""
        pt = self.get_unwrapped(instance)
        
        if raw: 
            # when would this happen? not sure, copying from TextField
            return pt.read()

        security = getSecurityManager()
        security.addContext(self)
        try:
            try:
                result = self.get_unwrapped(instance).pt_render(
                    extra_context={'user': security.getUser()})
            except TALESError, err:
                if err.type == 'Unauthorized':
                    raise err.type, err.value, err.traceback
                raise
            return result
        finally:
            security.removeContext(self)


__all__ = ( 'USAddressField'
          , 'LinkField'
          , 'USPhoneField'
          , 'EmailField'
          , 'InstructionField'
          , 'DynamicField'
          , 'SimpleDataGridWidget'
          )

